/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Clippings extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

var gClippingsList, gStatusBar;
var gCurrentListItemIndex = -1;
var gStrBundle;

// XPCOM RDF services
var gRDFSvc = gClippings.RDFSvc;
var gRDFContainerUtils = gClippings.RDFContainerUtils;

// Undo
var gUndoStack = {
  length: 0,
  _stack: [],

  push: function (aState)
  {
    this._stack.push(aState);
    this.length++;
  },

  pop: function ()
  {
    var rv = this._stack.pop();
    this.length--;
    return rv;
  }
};

// Redo - only 1 undo action is reversible
var gRedoStack = {
  length:   0,
  _lastUndo: null,

  push: function (aState)
  {
    this._lastUndo = aState;
    this.length = (this.length == 0 ? 1 : 1);
  },

  pop: function ()
  {
    var rv = {};
    for (var ppty in this._lastUndo) {
      rv[ppty] = this._lastUndo[ppty];
    }
    this._lastUndo = null;
    this.length = 0;
    return rv;
  }
};


// Flags for gUndoStack._stack[i].action
const ACTION_EDITNAME = 1;
const ACTION_EDITTEXT = 2;
const ACTION_DELETE = 3;
const ACTION_CREATENEW = 4;
const ACTION_CHANGEPOSITION = 5;

// Flags for aDestUndoStack parameter of functions for reversible actions
const UNDO_STACK = 1;
const REDO_STACK = 2;


// Drag and drop list box items to move them
var listitemObserver = {
  onDragStart: function (event, transferData, action)
  {
    var index = gClippingsList.selectedIndex;
    transferData.data = new TransferData();
    transferData.data.addDataForFlavour("text/unicode", index);
  },

  onDragOver: function (event, flavour, session)
  {
    event.target.style.border = "1px solid highlight";
  },

  onDragExit: function (event, session)
  {
    event.target.style.border = "none";
  },

  onDrop: function (event, transferData, session)
  {
    var oldIndex = parseInt(transferData.data);
    if (isNaN(oldIndex)) {
      return;
    }

    gClippingsList.selectedItem = event.target;
    var newIndex = gClippingsList.selectedIndex;
    moveEntry(oldIndex + 1, newIndex + 1, UNDO_STACK);
    gClippingsList.selectedIndex = newIndex;
    updateDisplay();
  },

  getSupportedFlavours: function ()
  {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/unicode");
    return flavours;
  }
};


// Window resize event handler

function resize(event)
{
  var wnd = document.getElementById("clippings-manager");
  var minWidth  = parseInt(wnd.getAttribute("minWidth"));
  var minHeight = parseInt(wnd.getAttribute("minHeight"));
  var width    = minWidth;
  var height   = minHeight;

  if (this.outerWidth >= minWidth) {
    width = this.outerWidth;
  }
  if (this.outerHeight >= minHeight) {
    height = this.outerHeight;
  }
  this.resizeTo(width, height);
}

window.addEventListener("resize", resize, false);



function init() 
{
  gStrBundle = document.getElementById("ae-clippings-strings");
  gClippingsList = document.getElementById("clippings-list");
  var rootNode = gRDFSvc.GetResource("http://clippings.mozdev.org/rdf/user-clippings");

  try {
    // If Clippings data file doesn't exist, it will be created automatically.
    gClippings.dataSrc = gRDFSvc.GetDataSourceBlocking(gClippings.getCurrentProfileURL() + "/" + CLIPDAT_FILE_NAME);
  }
  catch (e) {
    alert(e);
  }
  gClippingsList.database.AddDataSource(gClippings.dataSrc);

  try {
    // MakeSeq will create a new <RDF:Seq> if it doesn't already exist.
    // If it does, it just initializes it (similar to calling gClippings.RDFContainer.Init()).
    gClippings.RDFContainer = gRDFContainerUtils.MakeSeq(gClippings.dataSrc, rootNode);
    gClippings.RDFContainer = gClippings.RDFContainer.QueryInterface(Components.interfaces.nsIRDFContainer);
  }
  catch (e) {
    alert(e);
  }

  var numEntries = gClippings.RDFContainer.GetCount();
  document.getElementById("entry-properties").selectedIndex = numEntries == 0 ? 1 : 2;
    
  gClippingsList.builder.rebuild();
  gStatusBar = document.getElementById("app-status");
  gStatusBar.label = numEntries + " " + gStrBundle.getString('numEntriesPost');

  if (numEntries > 0) {
    gClippingsList.selectedIndex = 0;
    gCurrentListItemIndex = 0;
    gClippingsList.click();
  }

  gClippings.purgeTempData();

  // Periodically check for new entries created outside Clippings Manager 
  // window if the window is open in the background.
  window.setInterval(detectExternallyCreatedEntry, 500);
}


function detectExternallyCreatedEntry()
{
  var newEntryURI = gClippings.getPref("clippings.entries.tmp_data");
  if (newEntryURI) { 
    var newListItem = document.getElementById(newEntryURI);
    if (newListItem) {
      gClippingsList.selectItem(newListItem);
      gClippingsList.ensureElementIsVisible(newListItem);
      gClippingsList.click();

      var state = {
	action: ACTION_CREATENEW, 
	uri:    newEntryURI, 
	name:   newListItem.getAttribute("clipName"), 
	text:   newListItem.getAttribute("clipText")
      };
      gUndoStack.push(state);
      gClippings.log(String.format("New entry named %S added to undo stack (created outside Clippings Manager!!)", state.name));

      gClippings.resetPref("clippings.entries.tmp_data");
    }
  }
}


function newClipping() 
{
  updateCurrentClippingData();
  newClippingHelper(gStrBundle.getString("newEntryName"),"", null, UNDO_STACK);
}


function newClippingHelper(aName, aText, aURI, aDestUndoStack)
{
  var newNodeURI;
  if (aURI) {
    newNodeURI = gClippings.createNewClippingWithURI(aURI, aName, aText);
  }
  else {
    newNodeURI = gClippings.createNewClipping(aName, aText);
  }

  var newListItem;
  while (! (newListItem = document.getElementById(newNodeURI))) { 
    // Delay selection of the new entry until its corresponding <listitem>
    // element is created.
  }

  gClippingsList.selectItem(newListItem);
  gClippingsList.ensureElementIsVisible(newListItem);
  gClippingsList.click();

  var deck = document.getElementById("entry-properties");
  if (deck.selectedIndex != 0) {
    deck.selectedIndex = 0;
  }

  var clippingName = document.getElementById("clipping-name");
  clippingName.select();
  clippingName.focus();

  var state = {action: ACTION_CREATENEW, uri: newNodeURI, name: aName, text: aText};
  if (aDestUndoStack == UNDO_STACK) {
    gUndoStack.push(state);
    gClippings.log(String.format("New entry %S added to undo stack", aName));
  }
  else if (aDestUndoStack == REDO_STACK) {
    gRedoStack.push(state);
    gClippings.log(String.format("New entry %S added to redo stack", aName));
  }
}


function pasteClippingAsNew()
{
  updateCurrentClippingData();

  var newNodeURI = gClippings.newFromClipboard();
  if (! newNodeURI) {
    gClippings.beep();
    gStatusBar.label = gStrBundle.getString('errorNoClipboardContent');
    return;
  }

  var newListItem;
  while (! (newListItem = document.getElementById(newNodeURI))) { 
    // Delay selection of the new entry until its corresponding <listitem>
    // element is created.
  }
  
  gClippingsList.selectItem(newListItem);
  gClippingsList.ensureElementIsVisible(newListItem);
  gClippingsList.click();

  var deck = document.getElementById("entry-properties");
  if (deck.selectedIndex != 0) {
    deck.selectedIndex = 0;
  }

  var clippingName = document.getElementById("clipping-name");
  clippingName.select();
  clippingName.focus();

  gUndoStack.push({action: ACTION_CREATENEW, uri: newNodeURI, 
 		   name: newListItem.getAttribute("clipName"), 
	           text: newListItem.getAttribute("clipText")});
  gClippings.log(String.format("New entry added to undo stack\nName = %S", 
			       newListItem.getAttribute("clipName")));
}


function updateEditStatus() 
{
  gStatusBar.label = gStrBundle.getString("editEntry");
}


function saveClippings() 
{
  updateCurrentClippingData();
  gStatusBar.label = gStrBundle.getString("saveProgress");

  try {
    gClippings.flushDataSrc();
  }
  catch (e) {
    gStatusBar.label = gStrBundle.getString("errorSaveFailed");
    return;
  }

  gStatusBar.label = gStrBundle.getString("saveCompleted");
}


function deleteClipping() 
{
  if (gClippingsList.getRowCount() == 0) {
    gClippings.beep();
    gStatusBar.label = gStrBundle.getString("msgNothingToDelete");
    return;
  }

  updateCurrentClippingData();

  var currentItem = gClippingsList.selectedItem;
  if (! currentItem) {
    return;
  }

  deleteClippingHelper(currentItem, UNDO_STACK);
}


function deleteClippingHelper(aListItem, aDestUndoStack)
{
  var deletedItemIndex = gClippingsList.selectedIndex;
  var entry = {
    uri: aListItem.id,
    name: aListItem.getAttribute("clipName"),
    text: aListItem.getAttribute("clipText")
  };
  var subjectNode = gRDFSvc.GetResource(entry.uri);
  var predName = gRDFSvc.GetResource("http://clippings.mozdev.org/ns/rdf#name");
  var targName = gRDFSvc.GetLiteral(entry.name);
  var predText = gRDFSvc.GetResource("http://clippings.mozdev.org/ns/rdf#text");
  var targText = gRDFSvc.GetLiteral(entry.text);
  var state = {
    action: ACTION_DELETE, 
    uri:    entry.uri, 
    name:   entry.name, 
    text:   entry.text,
    pos:    deletedItemIndex + 1,
  };

  if (aDestUndoStack == UNDO_STACK) {
    gUndoStack.push(state);
    gClippings.log(String.format("Deleted entry %S (URI %s added to undo stack)\nPosition: %d", entry.name, entry.uri, state.pos));
  }
  else if (aDestUndoStack == REDO_STACK) {
    gRedoStack.push(state);
    gClippings.log(String.format("Deleted entry %S (URI %s added to redo stack)", entry.name, entry.uri));
  }

  gClippings.dataSrc.Unassert(subjectNode, predName, targName);
  gClippings.dataSrc.Unassert(subjectNode, predText, targText);
  gClippings.RDFContainer.RemoveElement(subjectNode, true);
  gClippingsList.builder.rebuild();

  var numEntries = gClippings.RDFContainer.GetCount();
  gStatusBar.label = numEntries + " " + gStrBundle.getString("numEntriesPost");

  var numItems = gClippingsList.getRowCount();
  if (numItems == 0) {
    document.getElementById("entry-properties").selectedIndex = 1;
    gCurrentListItemIndex = -1;
  }
  else if (deletedItemIndex == numItems) {  // Deleted entry was last list item
    gClippingsList.selectedIndex = numItems - 1;
    gClippingsList.ensureIndexIsVisible(numItems - 1);
    updateDisplay();
  }
  else {
    gClippingsList.selectedIndex = deletedItemIndex;
    gClippingsList.ensureIndexIsVisible(deletedItemIndex);
    updateDisplay();
  }
}


function updateCurrentEntryStatus()
{
  if (gClippingsList.getRowCount() == 0) {
    return;
  }

  var numEntries = gClippings.RDFContainer.GetCount();
  gStatusBar.label = numEntries + " " + gStrBundle.getString("numEntriesPost");

  var deck = document.getElementById("entry-properties");
  if (deck.selectedIndex != 0) {
    deck.selectedIndex = 0;
  }
}


function updateDisplay()
{
  var clippingName = document.getElementById("clipping-name");
  var clippingText = document.getElementById("clipping-text");
  
  clippingName.value = gClippingsList.selectedItem.getAttribute("clipName");
  clippingText.value = gClippingsList.selectedItem.getAttribute("clipText");

  gCurrentListItemIndex = gClippingsList.selectedIndex;

  gClippingsList.ensureIndexIsVisible(gClippingsList.selectedIndex);
}


function updateName(aName)
{
  if (gCurrentListItemIndex == -1) {
    return;
  }

  var currentItem = gClippingsList.getItemAtIndex(gCurrentListItemIndex);
  updateNameHelper(currentItem.id, aName, UNDO_STACK);
  currentItem.label = aName;
  currentItem.setAttribute("clipName", aName);
}


function updateNameHelper(aURI, aName, aDestUndoStack)
{
  var subjName = gRDFSvc.GetResource(aURI);
  var predName = gRDFSvc.GetResource("http://clippings.mozdev.org/ns/rdf#name");
  var oldName, targOldName;
  targOldName = gClippings.dataSrc.GetTarget(subjName, predName, true);

  if (targOldName instanceof Components.interfaces.nsIRDFLiteral) {
    oldName = targOldName.Value;
  }

  if (oldName == aName) {
    return;
  }

  var state = {action: ACTION_EDITNAME, uri: aURI, name: oldName, text: null};
  if (aDestUndoStack == UNDO_STACK) {
    gUndoStack.push(state);
    gClippings.log(String.format("In function updateNameHelper(): Old Entry name: %S\nOld entry name added to undo stack", oldName));
  }
  else if (aDestUndoStack == REDO_STACK) {
    gRedoStack.push(state);
    gClippings.log(String.format("In function updateNameHelper(): Old Entry name: %S\nOld entry name added to redo stack", oldName));
  }

  var targNewName = gRDFSvc.GetLiteral(aName);
  gClippings.dataSrc.Change(subjName, predName, targOldName, targNewName);
}


function updateText(aText)
{
  if (gCurrentListItemIndex == -1) {
    return;
  }

  var currentItem = gClippingsList.getItemAtIndex(gCurrentListItemIndex);
  updateTextHelper(currentItem.id, aText, UNDO_STACK);
  currentItem.setAttribute("clipText", aText);
}


function updateTextHelper(aURI, aText, aDestUndoStack)
{
  var subjText = gRDFSvc.GetResource(aURI);
  var predText = gRDFSvc.GetResource("http://clippings.mozdev.org/ns/rdf#text");
  var oldText, targOldText;
  targOldText = gClippings.dataSrc.GetTarget(subjText, predText, true);

  if (targOldText instanceof Components.interfaces.nsIRDFLiteral) {
    oldText = targOldText.Value;
  }

  if (oldText == aText) {
    return;
  }

  var state = {action: ACTION_EDITTEXT, uri: aURI, name: null, text: oldText};
  if (aDestUndoStack == UNDO_STACK) {
    // DEBUGGING
    var subjName = subjText;
    var predName = gRDFSvc.GetResource("http://clippings.mozdev.org/ns/rdf#name");
    var targName = gClippings.dataSrc.GetTarget(subjName, predName, true);
    var name = targName.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    // END DEBUGGING
    gUndoStack.push({action:ACTION_EDITTEXT, uri:aURI, name:name, text:oldText});
    // We shouldn't care what the old name is.
    // Non-debug invocation to push() should be as follows:
    /***
	gUndoStack.push(ACTION_EDITTEXT, aURI, null, oldText);
    ***/
    gClippings.log(String.format("In function updateTextHelper(): Entry name: %S\nOld text before edit added to undo stack\noldText = %S", name, oldText));
  }
  else if (aDestUndoStack == REDO_STACK) {
    gRedoStack.push(state);
    gClippings.log(String.format("In function updateTextHelper(): Added to redo stack: oldText == %S", oldText));
  }

  var targNewText = gRDFSvc.GetLiteral(aText);
  gClippings.dataSrc.Change(subjText, predText, targOldText, targNewText); 
}


function updateCurrentClippingData()
{
  if (gClippings.RDFContainer.GetCount() > 0) {
    updateName(document.getElementById("clipping-name").value);
    updateText(document.getElementById("clipping-text").value);
  }
}


function moveUp()
{
  var i = gClippingsList.selectedIndex;
  var numItems = gClippingsList.getRowCount();
  if (numItems != 0 && i != 0) {
    moveEntry(i + 1, i, UNDO_STACK);
    gClippingsList.selectedIndex = i - 1;
    gCurrentListItemIndex = gClippingsList.selectedIndex;
  }
}


function moveDown()
{
  var i = gClippingsList.selectedIndex;
  var numItems = gClippingsList.getRowCount();
  if (numItems != 0 && i != (numItems - 1)) {
    moveEntry(i + 1, i + 2, UNDO_STACK);
    gClippingsList.selectedIndex = i + 1;
    gCurrentListItemIndex = gClippingsList.selectedIndex;
  }
}


function moveEntry(aOldPos, aNewPos, aDestUndoStack)
  // aOldPos and aNewPos are 1-based (NOT zero-based) indices
{
  if (aDestUndoStack == UNDO_STACK) {
    gUndoStack.push({action: ACTION_CHANGEPOSITION, currPos: aNewPos, prevPos: aOldPos});
  }
  else if (aDestUndoStack == REDO_STACK) {
    gRedoStack.push({action: ACTION_CHANGEPOSITION, currPos: aNewPos, prevPos: aOldPos});
  }

  gClippings.log(String.format("In function moveEntry(): aOldPos=%d; aNewPos=%d", aOldPos, aNewPos));

  var node = gClippings.RDFContainer.RemoveElementAt(aOldPos, true);
  gClippings.RDFContainer.InsertElementAt(node, aNewPos, true);
}


function addClippingAtPosition(aName, aText, aURI, aPos)
{
  var newNodeURI;
  newNodeURI = gClippings.createNewClippingWithURI(aURI, aName, aText, aPos);
  
  var newListItem;
  while (! (newListItem = document.getElementById(newNodeURI))) { 
    // Delay selection of the new entry until its corresponding <listitem>
    // element is created.
  }
  gClippingsList.selectItem(newListItem);
  gClippingsList.ensureElementIsVisible(newListItem);
  gClippingsList.click();

  var deck = document.getElementById("entry-properties");
  if (deck.selectedIndex != 0) {
    deck.selectedIndex = 0;
  }
}


function initClippingsListPopup()
{
  var moveUp = document.getElementById("cmd_moveup");
  var moveDown = document.getElementById("cmd_movedown");
  var numListItems = gClippingsList.getRowCount();
  if (numListItems == 0) {
    return false;  // Don't show popup menu if list box is empty
  }
  else if (numListItems == 1) {
    moveUp.setAttribute("disabled", "true");
    moveDown.setAttribute("disabled", "true");
  }
  else if (gClippingsList.selectedIndex == 0) {
    moveUp.setAttribute("disabled", "true");
    moveDown.setAttribute("disabled", "false");
  }
  else if (gClippingsList.selectedIndex == (gClippingsList.getRowCount()-1)) {
    moveUp.setAttribute("disabled", "false");
    moveDown.setAttribute("disabled", "true");
  }
  else {
    moveUp.setAttribute("disabled", "false");
    moveDown.setAttribute("disabled", "false");
  }
  return true;
}


function undo()
{
  updateCurrentClippingData();

  if (gUndoStack.length == 0) {
    gClippings.beep();
    gStatusBar.label = gStrBundle.getString("msgNothingToUndo");
    return;
  }

  var undone = gUndoStack.pop();

  if (undone.action == ACTION_DELETE) {
    addClippingAtPosition(undone.name, undone.text, undone.uri, undone.pos);
    gCurrentListItemIndex = gClippingsList.selectedIndex;

    gClippings.log(String.format("Deletion undone\nEntry name: %S; entry URI: %S", undone.name, undone.uri));

    gRedoStack.push({action: ACTION_DELETE, uri: undone.uri, name: undone.name,
		     text: undone.text, pos: undone.pos});
  }
  else if (undone.action == ACTION_EDITTEXT) {
    updateTextHelper(undone.uri, undone.text, REDO_STACK);

    var listItem = selectListItemWithId(undone.uri);
    if (listItem) {
      updateDisplay();

      var clippingText = document.getElementById("clipping-text");
      clippingText.focus();
      clippingText.select();

      gClippings.log(String.format("Entry text edit undone\nEntry name: %S; entry URI: %S; oldText: %S", undone.name, undone.uri, undone.text));
    }
  }
  else if (undone.action == ACTION_EDITNAME) {
    updateNameHelper(undone.uri, undone.name, REDO_STACK);

    var listItem = selectListItemWithId(undone.uri);
    if (listItem) {
      updateDisplay();
      listItem.label = undone.name;

      var clippingName = document.getElementById("clipping-name");
      clippingName.focus();
      clippingName.select();

      gClippings.log(String.format("Entry name edit undone\nEntry name: %S; entry URI:", undone.name, undone.uri));
    }
  }
  else if (undone.action == ACTION_CREATENEW) {
    var listItem = selectListItemWithId(undone.uri);
    deleteClippingHelper(listItem);
    gClippings.log(String.format("Entry creation undone\nEntry name: %S; entry URI: %S", undone.name, undone.uri));
    gRedoStack.push({action: ACTION_CREATENEW, uri: undone.uri, 
		     name: undone.name, text: undone.text});
  }
  else if (undone.action == ACTION_CHANGEPOSITION) {
    moveEntry(undone.currPos, undone.prevPos, REDO_STACK);
    gClippingsList.selectedIndex = undone.prevPos - 1;
    updateDisplay();

    gClippings.log(String.format("Reversed entry move; current selection: %S\nAt index: %d\ngCurrentListItemIndex: %d", gClippingsList.selectedItem.label, gClippingsList.selectedIndex, gCurrentListItemIndex));
  }
}


function selectListItemWithId(aID)
{
  var rv; 
  var listItems = gClippingsList.getElementsByAttribute("id", aID);

  if (listItems) {
    var listItem = listItems[0];
    gClippingsList.selectItem(listItem);
    gClippingsList.ensureElementIsVisible(listItem);
    rv = listItem;
  }
  return rv;
}


function reverseLastUndo()
{
  updateCurrentClippingData();

  if (gRedoStack.length == 0) {
    gClippings.beep();
    return;
  }

  var lastUndo = gRedoStack.pop();

  if (lastUndo.action == ACTION_DELETE) {
    var listItem = selectListItemWithId(lastUndo.uri);
    deleteClippingHelper(listItem, UNDO_STACK);
    gClippings.log(String.format("Entry deletion redone!\nEntry name: %S; entry URI: %S", lastUndo.name, lastUndo.uri));
  }
  else if (lastUndo.action == ACTION_EDITNAME) {
    updateNameHelper(lastUndo.uri, lastUndo.name, UNDO_STACK);

    var listItem = selectListItemWithId(lastUndo.uri);
    if (listItem) {
      updateDisplay();
      listItem.label = lastUndo.name;

      var clippingName = document.getElementById("clipping-name");
      clippingName.focus();
      clippingName.select();

      gClippings.log(String.format("Entry name edit redone!\nEntry name: %S; entry URI:", lastUndo.name, lastUndo.uri));
    }
  }
  else if (lastUndo.action == ACTION_EDITTEXT) {
    updateTextHelper(lastUndo.uri, lastUndo.text, UNDO_STACK);

    var listItem = selectListItemWithId(lastUndo.uri);
    if (listItem) {
      updateDisplay();

      var clippingText = document.getElementById("clipping-text");
      clippingText.focus();
      clippingText.select();

      gClippings.log(String.format("Entry text edit redone!\nEntry name: %S; entry URI: %S; oldText: %S", lastUndo.name, lastUndo.uri, lastUndo.text));
    }
  }
  else if (lastUndo.action == ACTION_CREATENEW) {
    newClippingHelper(lastUndo.name, lastUndo.text, lastUndo.uri, UNDO_STACK);
    selectListItemWithId(lastUndo.uri);
    gClippings.log(String.format("Deletion redone!\nEntry name: %S; entry URI: %S", lastUndo.name, lastUndo.uri));
  }
  else if (lastUndo.action == ACTION_CHANGEPOSITION) {
    moveEntry(lastUndo.currPos, lastUndo.prevPos, UNDO_STACK);
    gClippingsList.selectedIndex = lastUndo.prevPos - 1;
    updateDisplay();
  }

  gStatusBar.label = gStrBundle.getString("redoStatus");
}


function showHelp() 
{
  updateCurrentClippingData();
  alert(gStrBundle.getString("helpMsg"));
}
